from typing import Dict, List, Optional

from psutil import process_iter, Process


def parse_cmdline_args(cmdline_args) -> Dict[str, str]:
    cmdline_args_parsed = {}
    for cmdline_arg in cmdline_args:
        if len(cmdline_arg) > 0 and '=' in cmdline_arg:
            key, value = cmdline_arg[2:].split('=')
            cmdline_args_parsed[key] = value
    return cmdline_args_parsed


def return_process(process_name: List[str]) -> Optional[Process]:
    for process in process_iter():
        if process.name() in process_name:
            return process
    return None


def _return_ux_process_when_available() -> Process:
    process = None
    while not process:
        process = return_process(['LeagueClientUx.exe', 'LeagueClientUx'])
    return process



import asyncio
import logging
from abc import ABC
from sys import platform
from typing import Union

from psutil import Process

from .connection import Connection
from .events.managers import ConnectorEventManager, WebsocketEventManager
from .utils import _return_ux_process_when_available

logger = logging.getLogger('lcu-driver')


class BaseConnector(ConnectorEventManager):
    def __init__(self, loop=None):
        super().__init__()
        self.loop = loop or asyncio.get_event_loop()
        self.ws = WebsocketEventManager()
        self.connection = None

    def create_connection(self, process_or_string: Union[Process, str]):
        """Creates a connection and saves a reference to it"""
        connection = Connection(self, process_or_string)
        self.connection = connection

    def remove_connection(self):
        """Cancel the connection"""
        self.connection = None


class Connector(BaseConnector):
    def __init__(self, *, loop=None):
        super().__init__(loop)
        self._repeat_flag = True

    def start(self) -> None:
        """Starts the connector. This method should be overridden if different behavior is required.

        :rtype: None
        """
        try:
            def wrapper():
                connection = _return_ux_process_when_available()
                self.create_connection(connection)
                self.loop.run_until_complete(self.connection.init())

                if self._repeat_flag and len(self.ws.registered_uris) > 0:
                    logger.debug('Repeat flag=True. Looking for new clients.')
                    wrapper()

            wrapper()
        except KeyboardInterrupt:
            logger.info('Event loop interrupted by keyboard')

    async def stop(self) -> None:
        """Flag the connector to don't look for more clients once the connection finishes his job.

        :rtype: None
        """
        self._repeat_flag = False



class PlatformNotSupported(BaseException):
    pass


class IntegrityError(BaseException):
    pass


class CoroutineExpected(BaseException):
    def __init__(self, coroutine_name):
        super().__init__(f'a coroutine was expected, replace "def {coroutine_name}" by "async def {coroutine_name}"')


class EarlyPerform(BaseException):
    pass


class InvalidURI(BaseException):
    def __init__(self, error_type, used_uri=None):
        if error_type == 'backslash':
            super().__init__(f'every endpoint must start with a backslash, replace {used_uri} by /{used_uri}')
